"""
Lily unit test package
"""

from lily_unit_test.models.classification import Classification
from lily_unit_test.models.test_runner import TestRunner
from lily_unit_test.models.test_suite import TestSuite

# For easy import:
Classification = Classification
TestRunner = TestRunner
TestSuite = TestSuite
